from suds.client import Client
from suds.xsd.doctor import Import, ImportDoctor
from suds.wsse import *
import datetime

class SpiraModerator(object):
	"""
	Helper class to connect and manipulate the data in SpiraTest
	"""

	client = None 						# Soap client to connect to Spira Soap API 
	username = "ypgAutoTester"			# Automation User username (restricted access to SpiraTest)
	password = "@ut0t3$t3r"				# Automation User password
	tests = None 						# Holds retrieved tests from Automation test set in Spiratest


	def __init__(self):
		"""
		Constructs client to connect to SpiraTest Soap API
		"""
		WDSLFile = "http://mtljiraqcprod01.ad.ypg.com/SpiraTeam/Services/v4_0/ImportExport.svc?wsdl"
		schema_url = 'http://schemas.microsoft.com/2003/10/Serialization/Arrays'
		schema_import = Import(schema_url)
		schema_doctor = ImportDoctor(schema_import)

		self.client = Client(WDSLFile,doctor=schema_doctor)


	def testConnect(self):
		"""
		Authenticates client with SpiraTest.  

		Returns : (bool) if connection was successful
		"""

		return self.client.service.Connection_Authenticate(self.username,self.password)


	def updateTestCase(self,projectId,testCaseId,testStart,testEnd,testStatusId,runnerMsg,stackTrace):
		"""
		Updates a specified test cases with a specified projects Automation test set with the results of the test execution

		Args : 
		projectId : (int) Unique project id from within SpiraTest
		testCaseId : (int) Unique test case id from within SpiraTest
		testStart : (datetime) Time at which the test case started its execution. Doesn't seem to affect Spiratest.
		testEnd : (datetime) Time at which the test case ended its execution. Doesn't seem to affect Spiratest.
		runnerMsg : (str) Parameters that were used in test execution
		stackTrace : (str) Error message returned from failed test cases

		"""

		self.client.service.TestRun_RecordAutomated2(self.username, self.password, projectId, None, testCaseId, 
													None, 1, None, None, testStart, testEnd, testStatusId, 
													"None", None, 1, runnerMsg, stackTrace, 1 )

	def getProjectId(self,projName):
		"""
		Gets the Unique prject id from spiratest for a given project name.  Name comes from project's config file

		Args : projName : (str) Name of project under test 

		Returns : (int) project id
		"""

		self.client.service.Connection_Authenticate(self.username,self.password)
		projectList = self.client.service.Project_Retrieve()
		project = (proj for proj in projectList[0] if proj["Name"] == projName).next()
		
		return project.ProjectId

	def getTestsFromSet(self,testSetId, projId):
		"""
		Fetches a list of test cases for a given test set id in SpiraTest and stores them in class var "tests"

		Args : 
		testSetId : (int) Automation test set id from Spiratest for project under test
		projId : (int) project id from Spiratest for project under test

		"""

		self.client.service.Connection_Authenticate(self.username,self.password)
		self.client.service.Connection_ConnectToProject(projId)
		testSet = self.client.service.TestCase_RetrieveByTestSetId(testSetId)
		self.tests = testSet[0]

	def getTestCaseId(self,testName):
		"""
		Gets the unique test case id from SpiraTest for the test case name supplied

		Args : testName : (str) Test case name found in SpiraTest and used in test case decorated to label automated test_support

		Returns : (int) test case id 
		"""

		myTest = (test for test in self.tests if test["Name"] == testName).next()

		return myTest.TestCaseId
		









